/*
 * (C) Copyright 2000
 * DENX Software Engineering
 * Wolfgang Denk, wd@denx.de
 * All rights reserved.
 *     
 * $Date: $
 * $Revision: $
 */

#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <linux/flash.h>


int main (int argc, char *argv[])
{
	int i, fd, part;
	struct stat stb;
	flash_info_t info;
	flash_region_info_t *region;
	flash_partition_t *partition;

	if (argc != 2) {
		fprintf (stderr, "Usage: %s flash_device\n", *argv);
		return (EXIT_FAILURE);
	}

	if ((fd = open(argv[1], O_RDONLY)) < 0) {
		fprintf (stderr, "%s: Can't open %s: %s\n",
			*argv, argv[1], strerror(errno));
		return (EXIT_FAILURE);
	}

	if (fstat(fd, &stb) < 0) {
		fprintf (stderr, "%s: Can't stat %s: %s\n",
			*argv, argv[1], strerror(errno));
		return (EXIT_FAILURE);
	}

	part = stb.st_rdev & FLASH_PART_MASK;

	if (ioctl (fd, FLASHGETINFO, &info) != 0) {
		fprintf (stderr, "%s: FLASHGETINFO failed: %s\n",
			*argv, strerror(errno));
		return (EXIT_FAILURE);
	}

	printf ("Information for %s", argv[1]);
	if (part) {
		printf (" = Partition %d on this device", part);
	}
	printf (":\n\nSize: %2d MB,  Configuration: %dx%d bit,"
		"  %d Regions,  %d Partitions\n\n",
		info.size >> 20, info.chip_config, info.bitwidth,
		info.num_regions, info.num_parts);

	if ((region = malloc (sizeof (flash_region_info_t) * info.num_regions))
		== NULL) {
		fprintf (stderr, "%s: Can't malloc %d bytes: %s\n",
			*argv,
			sizeof (flash_region_info_t) * info.num_regions,
			strerror(errno));
		return (EXIT_FAILURE);
	}

	if (ioctl (fd, FLASHGETREGIONINFO, region) != 0) {
		fprintf (stderr, "%s: FLASHGETREGIONINFO failed: %s\n",
			*argv, strerror(errno));
		return (EXIT_FAILURE);
	}

	for (i = 0; i < info.num_regions; i++) {
		printf ("Region    %d:  Offset: 0x%08lx"
			"  Size: %5d kB in %3d Sectors of %4d kB each\n",
			i,
			region[i].offset,
			region[i].num_sectors * region[i].sector_size >> 10,
			region[i].num_sectors,
			region[i].sector_size >> 10
		);
	}

	if (!part) {
		(void)close (fd);
		return (EXIT_SUCCESS);
	}

	if ((partition = malloc (sizeof (flash_partition_t) * info.num_parts))
		== NULL) {
		fprintf (stderr, "%s: Can't malloc %d bytes: %s\n",
			*argv,
			sizeof (flash_partition_t) * info.num_parts,
			strerror(errno));
		return (EXIT_FAILURE);
	}

	if (ioctl (fd, FLASHGETPARTINFO, partition) != 0) {
		fprintf (stderr, "%s: FLASHGETPARTINFO failed: %s\n",
			*argv, strerror(errno));
		return (EXIT_FAILURE);
	}

	printf ("\n");

	for (i = 0; i < info.num_parts; i++) {
		printf ("Partition %d:  Offset: 0x%08lx  Size: %5d kB\n",
			i+1,
			partition[i].start,
			partition[i].size >> 10
		);
	}

	(void)close (fd);
	return (EXIT_SUCCESS);
}
